@@ -5,7 +5,7 @@
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
 
-      http://www.apache.org/licenses/LICENSE-2.0
+      https://www.apache.org/licenses/LICENSE-2.0
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -19,6 +19,7 @@
 #include "Utils.h"
 
 #include "clang/AST/ASTContext.h"
+#include "clang/AST/Attr.h"
 #include "clang/AST/Decl.h"
 #include "clang/AST/DeclCXX.h"
 #include "clang/AST/DeclFriend.h"
@@ -189,7 +190,11 @@ protected:
   {                                                                           \
     this->OutputUnimplementedType(t, dn);                                     \
   }
-#include "clang/AST/TypeNodes.def"
+#if LLVM_VERSION_MAJOR >= 10
+#  include "clang/AST/TypeNodes.inc"
+#else
+#  include "clang/AST/TypeNodes.def"
+#endif
 
   void OutputUnimplementedType(clang::Type const* t, DumpNode const* dn)
   {
@@ -393,6 +398,7 @@ class ASTVisitor : public ASTVisitorBase
 
   /** Print a name="..." attribute.  */
   void PrintNameAttribute(std::string const& name);
+  void PrintNameAttribute(llvm::StringRef name);
 
   /** Print a mangled="..." attribute.  */
   void PrintMangledAttribute(clang::NamedDecl const* d);
@@ -744,14 +750,10 @@ ASTVisitor::DumpId ASTVisitor::AddTypeDumpNode(DumpType dt, bool complete,
         DumpType(t->getAs<clang::DecayedType>()->getDecayedType(), c),
         complete, dq);
     case clang::Type::Decltype:
-      if (this->Opts.CastXml && t->isNullPtrType()) {
+      if (this->Opts.CastXml) {
         clang::DecltypeType const* dtt = t->getAs<clang::DecltypeType>();
-        if (dtt->getUnderlyingExpr()->getStmtClass() ==
-            clang::Stmt::CXXNullPtrLiteralExprClass) {
-          // Treat literal 'decltype(nullptr)' as a FundamentalType.
-          return this->AddTypeDumpNode(DumpType(dtt->getUnderlyingType(), c),
-                                       complete, dq);
-        }
+        return this->AddTypeDumpNode(DumpType(dtt->getUnderlyingType(), c),
+                                     complete, dq);
       }
       break;
     case clang::Type::Elaborated:
@@ -1078,7 +1080,7 @@ void ASTVisitor::ProcessFileQueue()
     this->OS <<
       "  <File"
       " id=\"f" << this->FileNodes[f] << "\""
-      " name=\"" << encodeXML(f->getName()) << "\""
+      " name=\"" << encodeXML(std::string(f->getName())) << "\""
       "/>\n"
       ;
     /* clang-format on */
@@ -1115,7 +1117,11 @@ void ASTVisitor::OutputType(DumpType dt, DumpNode const* dn)
     this->Output##CLASS##Type(                                                \
       static_cast<clang::CLASS##Type const*>(t.getTypePtr()), dn);            \
     break;
-#include "clang/AST/TypeNodes.def"
+#if LLVM_VERSION_MAJOR >= 10
+#  include "clang/AST/TypeNodes.inc"
+#else
+#  include "clang/AST/TypeNodes.def"
+#endif
     }
   }
 }
@@ -1186,6 +1192,11 @@ void ASTVisitor::PrintNameAttribute(std::string const& name)
   this->OS << " name=\"" << encodeXML(n) << "\"";
 }
 
+void ASTVisitor::PrintNameAttribute(llvm::StringRef name)
+{
+  this->PrintNameAttribute(std::string(name));
+}
+
 void ASTVisitor::PrintMangledAttribute(clang::NamedDecl const* d)
 {
   // Compute the mangled name.
@@ -1710,7 +1721,7 @@ void ASTVisitor::OutputTranslationUnitDecl(clang::TranslationUnitDecl const* d,
 {
   this->OS << "  <Namespace";
   this->PrintIdAttribute(dn);
-  this->PrintNameAttribute("::");
+  this->PrintNameAttribute(std::string("::"));
   if (dn->Complete) {
     this->PrintMembersAttribute(d);
   }
@@ -1891,6 +1902,9 @@ void ASTVisitor::OutputEnumDecl(clang::EnumDecl const* d, DumpNode const* dn)
   this->PrintNameAttribute(name);
   this->PrintContextAttribute(d);
   this->PrintLocationAttribute(d);
+  if (this->Opts.CastXml && d->isScoped()) {
+    this->OS << " scoped=\"1\"";
+  }
   this->PrintABIAttributes(d);
   this->PrintAttributesAttribute(d);
   clang::EnumDecl::enumerator_iterator enum_begin = d->enumerator_begin();
@@ -2215,7 +2229,7 @@ void ASTVisitor::OutputStartXMLTags()
     // Start dump with castxml-compatible format.
     /* clang-format off */
     this->OS <<
-      "<CastXML format=\"" << Opts.CastXmlEpicFormatVersion << ".1.5\">\n"
+      "<CastXML format=\"" << Opts.CastXmlEpicFormatVersion << ".1.6\">\n"
       ;
     /* clang-format on */
   } else if (this->Opts.GccXml) {
